@@ -24,10 +24,13 @@ from megadetector.detection.run_detector import get_typical_confidence_threshold
 from megadetector.utils.ct_utils import get_max_conf
 from megadetector.utils import write_html_image_list
 from megadetector.utils.path_utils import path_is_abs
-from megadetector.utils.wi_utils import load_md_or_speciesnet_file
+from megadetector.utils.path_utils import open_file
+from megadetector.utils.wi_taxonomy_utils import load_md_or_speciesnet_file
 from megadetector.visualization import visualization_utils as vis_utils
 from megadetector.visualization.visualization_utils import blur_detections
 
+default_box_sort_order = 'confidence'
+
 
 #%% Constants
 
@@ -49,7 +52,7 @@ def _render_image(entry,
                   out_dir,
                   images_dir,
                   output_image_width,
-                  box_sort_order=None,
+                  box_sort_order=default_box_sort_order,
                   category_names_to_blur=None):
     """
     Internal function for rendering a single image.
@@ -153,7 +156,7 @@ def visualize_detector_output(detector_output_path,
                               parallelize_rendering=False,
                               parallelize_rendering_n_cores=10,
                               parallelize_rendering_with_threads=True,
-                              box_sort_order=None,
+                              box_sort_order=default_box_sort_order,
                               category_names_to_blur=None):
     """
     Draws bounding boxes on images given the output of a detector.
@@ -397,7 +400,7 @@ def main(): # noqa
         '--category_names_to_blur', default=None, type=str,
         help='Comma-separated list of category names to blur (or a single category name, typically "person")')
     parser.add_argument(
-        '--classification_confidence', type=float, default=0.1,
+        '--classification_confidence', type=float, default=0.3,
         help='If classification results are present, render results above this threshold')
 
     if len(sys.argv[1:]) == 0:
@@ -424,8 +427,8 @@ def main(): # noqa
         html_output_file=args.html_output_file,
         category_names_to_blur=category_names_to_blur)
 
-    if args.html_output_file is not None and args.open_html_output_file:
-        from megadetector.utils.path_utils import open_file
+    if (args.html_output_file is not None) and args.open_html_output_file:
+        print('Opening output file {}'.format(args.html_output_file))
         open_file(args.html_output_file)
 
 if __name__ == '__main__':
